#This script contains a function for fitting a standard DetGP (just for the initial plots, there's plenty of good software to do this instead)

DetGPFit <- function(x, y) {
  #This function fits the DetGP to the outputs
  
  #needs input points "x", and outputs "y" from a deterministic simulator
  
  #the output of this function is a list of everything fit in stan
  
  library(MASS)
  library(Matrix)
  library(rstan)
  library(rdist)
  
  #location of Stan file used for parameter estimation
  StanLoc <- "DetGP.stan"
  
  dimen <- length(x[1,])    #get dimension of problem
  N <- length(x[,1])        #get number of data points for stochastic simulator
  
  #computational nugget
  nugget = 0.0001
  
  
  #H matrix (for mean function) (this can be customised, but changes need to replicated in the predict function)
  ##HetGP
  H <- function(x){
    #creates our "H" matrix for HetGP
    cbind(1,x) #linear mean function
  }
  Hmatrix <- H(x)               #get H matrix
  dimenH <- length(Hmatrix[1,]) #How many dimensions is it?
  
  
  
  #Prepare everything for Stan
  #initialise data
  data <- list(N = N, d = dimen, y = array(y, dim = N), x = x, nugget = nugget, dH = dimenH, H = Hmatrix)
  
  #initialise Stan model
  Detmodel <- stan_model(file=StanLoc)
  
  
  
  
  #Run optimiser
  #Sometimes the optimiser fails for various reasons
  #This is very rare, but given so many fits of this model were needed in the paper for simulation studies
  #even 1/100 failures would output an NA a set of results
  #The folloiwng lines of code rerun a failed optimisation 5 times, if it still fails, a different optimiser is run
  
  #run optimiser up to five times 
  opt <- NULL #start assuming optimiser will fail (unlikely)
  attempt <- 1 #what attempt are we (dont want to just keep trying, only try 5 times maximum)
  while( is.null(opt) && attempt <= 5 ) {
    attempt <- attempt + 1
    try( #attempt to run the optimiser
      errors <- capture.output(opt <- optimizing(Detmodel, data = data, as_vector = FALSE, iter = 10000, verbose = TRUE))
    )
    if(grepl("failed", errors[length(errors)]) | grepl("iterations", errors[length(errors)])) { #check to see if optimiser actually worked
      opt <- NULL #if optimiser threw some kind of error, dont save any of the results, and try again
    }
  } 
  
  if(is.null(opt)){ #if it still fails 5 times in a row, run a different optimiser thats a bit slower
    opt <- optimizing(Detmodel, data = data, as_vector = FALSE, iter = 2000, algorithm = "Newton", verbose = TRUE)
  }
  
  
  
  
  #Extract MAP estimates
  parameterestimates <- opt$par
}




#here is an example line of code running this function
#parameterestimates <- DetGPFit(xdet, ydet)