#This script contains a function for predicting from a DetGP

DetGPPredict <- function(x, y, parameterestimates, xpred) {
  #This function predicts from DetGP
  
  #needs input points "x", and outputs "y" from a deterministic simulator
  #And points to predict for xpred
  #Also needs a list "parameterestimates" (output from DetGPFit)
  
  #The output is a list, the first object is the posterior means, and the second is the posterior variances
  
  library(MASS)
  library(Matrix)
  library(rstan)
  library(rdist)
  
  #we want predictions for new points, but to do this we need a few functions
  
  #we need the squared exponential covariance function (these *could* be customised, but would require a change in Stan as well, and probably would be parametrised differently, i.e. no length scale)
  K12 <-  function(X1, X2, l) { 
    #Calculates squared exponential covariance matrix for given inputs
    n1 = length(X1[,1]) #number of input points
    n2 = length(X2[,1]) #number of validation points
    d = length(X1[1,])  #number of variables
    D = as.matrix(cdist(scale(X1,center=FALSE,scale=l),scale(X2,center=FALSE,scale=l))) 
    return(exp(-(D^2)))
  }
  
  
  #H matrix (for mean function) (this can be customised, but changes need to replicated in the fit function)
  ##HetGP
  H <- function(x){
    #creates our "H" matrix for HetGP
    cbind(1,x) #linear mean function
  }
  Hmatrix <- H(x)               #get H matrix
  dimenH <- length(Hmatrix[1,]) #How many dimensions is it?
  
  
  #Then given a H matrix, obtain the mean function values
  meanfunc <-  function(H, beta) {
    #Calculates (prior) mean function for a given GP
    H%*%as.vector(beta)
  }
  

  #this function then obtains predictions from the GP
  yGP <- function(inputx, inputy, predx, alpha, beta, l, nugget) {
    #generates a gaussian process for given inputs, outputs, prediction points, and parameter estimates
    #This GP will be for the deterministic output y
    
    #We need A = K(X, X); Kyx = K(X*, X); Kyy = K(X*, X*); Kxy = K(X, X*)

    A <- (alpha^2) * K12(inputx, inputx, l) + diag(rep(nugget, length(inputx[,1])), nrow = length(inputx[,1])) #plus nugget
    Ainv <- chol2inv(chol(A))
    
    Kyx <- (alpha^2) * K12(predx, inputx, l)
    Kxy <- t(Kyx)
    Kyy <- (alpha^2) * K12(predx, predx, l) + diag(rep(nugget, length(predx[,1])), nrow = length(predx[,1]))  #plus nugget and delta
    
    #Now get posterior mean 'function'
    #regression matrix
    Hpred <- H(predx)
    Hinput <- H(inputx)
    postmean <- meanfunc(Hpred, beta) +Kyx%*%Ainv%*%(inputy - meanfunc(Hinput, beta)) #calculate posterior mean
    
    #And posterior covariance 'function'
    postcov <- Kyy - Kyx%*%Ainv%*%Kxy #calculate posterior covariance
    
    GP <- list(postmean = postmean, postcov = postcov) #return the posterior mean and covariance
  }
  
  
  
  #make predictions
  nugget <- 0.0001 #nugget
  GPfuncs <- yGP(x, y, xpred, parameterestimates$alpha, as.vector(parameterestimates$beta), as.vector(parameterestimates$length_scale), nugget)

  return(list(mean = GPfuncs$postmean, var = diag(GPfuncs$postcov)))
  
  
}


#here is an example line(s) of code running this function
#GPpred <- DetGPPredict(xdet,ydet, parameterestimates, xpred)




